/*
 	Copyright (C) 2001, 2002 Artem V. Andreev (artem@AA5779.spb.edu)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef MODULES_H
#define MODULES_H

#ifdef USE_LTDL
#include <ltdl.h>
#else
typedef void *lt_dlhandle; // dummy
#endif
#include "lists.h"
#include "interp.h"

DEFLIST(library,
		char *name;
		lt_dlhandle handle;
		);

DEFLIST(mod_attach,
		interpreter *i;
		void *data;
		);

DEFLIST(module,
		char *name;
		struct module_map *map_ptr;
		lt_dlhandle handle;
		mod_attach *attached;
		);

DEFLIST(denied_module,
		interpreter *i;
		char *name;);

typedef void (*mod_init_f)(module *mod);
typedef void (*mod_attach_f)(mod_attach *ma);

#define SL_MOD_PATH_ENV ("SL_MOD_PATH")
#define SL_SCM_PATH_ENV ("SL_SCM_PATH")
#define DEFAULT_MOD_PATH "/usr/local/lib/nsl/modules"

int module_in_map(char *name);
library *load_library(char *name);
module *load_module(char *name);
void deny_module(interpreter *i, char *name, int flags);
enum deny_module_flags { deny_none, deny_loaded, deny_loading };

void attach_module(interpreter *i, char *name);

void init_modules(char *dlft);

void *get_mod_sym(char *modname, char *sym);
void *get_ma_data(module *mod, interpreter *i);

void append_builtins(interpreter *i, master_instruction_src *set);
void replace_builtins(interpreter *i, master_instruction_src *rep, instruction_t *saved);

void module_pragma(interpreter *i, char *name, char *pragma);
const char *module_version(char *name);
const char *module_date(char *name);
const char *module_srcdate(char *name);

#endif
